;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Fishin' Boo Boss, by andy_k_250					     ;;
;;									     ;;
;; Description: A Fishin' Boo that can only be defeated with thrown objects. ;;
;; Uses GFX06 in SP3 and GFX11 in SP4 for its graphics by default.	     ;;
;;									     ;;
;; Special Thanks To: Davros, smkdan, MarioEdit				     ;;
;;									     ;;
;; Based On The Following Sprite:					     ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Fishin' Boo, adapted by Davros. ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TEMP = $09

;hitpoints
HITPOINTS = $04
;enemy hit sound
ENEMYHITSOUND = $28
;extra bit for secret goal
EXTRA_BITS = $7FAB10

;1528: hitpoints
;1570: flash timer
;1602: original palette

FLASHTIMER = $40

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite initialization JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "INIT"

	LDA $15F6,x		;store original pal
	STA $1602,x
	LDA #HITPOINTS		;store hitpoints
	STA $1528,x

                    RTL
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            
                    dcb "MAIN"                        
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR START_SPRITE_CODE   ;  |
                    PLB                     ;  |
                    RTL                     ; /
                   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


X_ACCEL		    dcb $01,$FF		
X_SPEED		    dcb $10,$F0 
Y_ACCEL		    dcb $01,$FF			
Y_SPEED		    dcb $20,$E0
		

START_SPRITE_CODE   JSR SUB_GFX             ; sprite graphics routine

                    LDA $9D                 ; \ if sprites locked, return
                    BNE RETURN              ; /

		    JSL $01A7DC             ; check for Mario/sprite contact

		    JSR SUB_HORZ_POS        ; horizontal Mario/sprite check routine
		    STZ $1602,x             ; restore image
		    LDA $15AC,x             ; \ if the turn timer is set...
		    BEQ NO_DIRECTION        ; /
		    INC $1602,x             ; increase image
		    CMP #$10                ; \ if the image is set...
		    BNE NO_DIRECTION        ; /
		    TYA                     ; transfer to...
		    STA $157C,x             ; direction

RETURN		    RTS

NO_DIRECTION	    TXA
		    ASL A
		    ASL A
		    ASL A
		    ASL A
		    ADC $13                 ; \ if the frame is set...
		    AND #$3F                ; /
		    ORA $15AC,x             ; \ or the turn timer is set...
		    BNE NO_TIMER            ; /
		    LDA #$20                ; \ set time to turn
		    STA $15AC,x             ; /

NO_TIMER	    LDA $18BF
		    BEQ SET_X_SPEED
		    TYA
		    EOR #$01
		    TAY

SET_X_SPEED	    LDA $B6,x               ; \ compare x speed     
		    CMP X_SPEED,y           ;  |
		    BEQ CALC_FRAME          ;  |
		    CLC                     ;  |
		    ADC X_ACCEL,y           ;  | set up acceleration
		    STA $B6,x               ; /

CALC_FRAME	    LDA $13                 ; \ if the frame is set...
		    AND #$01                ;  |
		    BNE HORZ_SPEED          ; /

		    LDA $C2,x               ; \ set state
		    AND #$01                ;  |
		    TAY                     ; /
		    LDA $AA,x               ; \
		    CLC                     ;  |
		    ADC Y_ACCEL,y           ;  | set up acceleration
		    STA $AA,x               ;  |
		    CMP Y_SPEED,y           ;  | compare x speed
		    BNE HORZ_SPEED          ; /
		    INC $C2,x               ; increase sprite state

HORZ_SPEED	    LDA $B6,x
		    PHA
		    LDY $18BF
		    BNE UPDATE_POSITION
		    LDA $17BD
		    ASL A
		    ASL A
		    ASL A
		    CLC
		    ADC $B6,x
		    STA $B6,x
UPDATE_POSITION	    JSL $018022             ; update x speed without y gravity
		    PLA
		    STA $B6,x

UPDATE_SPEED        JSL $01801A             ; update speed based on position

		    JSR FLAME_INTERACT      ; flame interaction routine

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	LDA $1602,x		;palette which may get overwritten
	STA $15F6,x

	LDA $1570,x		;load flash timer
	BEQ CheckSprite
	JSR Flash		;keep flashing for timer's duration
	DEC $1570,x		;decrement timer

CheckSprite
	STX TEMP		;store self to TEMP
	LDY #$0B		;12 entries
CheckLoop
	CPY TEMP		;skip self
	BEQ NextSprite

	LDA $14C8,y		;check status
	CMP #$09
	BCC NextSprite		;less than 8 = next sprite
	
	LDA $1686,Y		;skip sprites that don't interact with others
	AND #$08
	BNE NextSprite

	JSL $03B69F		;clipA
	PHX
	TYX
	JSL $03B6E5		;clipB
	PLX
	JSL $03B72B		;check contact between A and B
	BCC NextSprite		;no contact = don't set

	LDA #$00
	STA $14C8,y		;erase sprite

	LDA $E4,x		;preserve xpos
	PHA
	LDA $D8,x		;preserve ypos
	PHA
	LDA $E4,y		;sprite ypos
	STA $E4,x
	LDA $D8,y		;sprite xpos
	STA $D8,x

	JSR SUB_SMOKE		;and smoke
	
	LDA #FLASHTIMER		;time to flash for
	STA $1570,x

	PLA
	STA $D8,x		;restore ypos
	PLA
	STA $E4,x		;restore xpos

	LDA #ENEMYHITSOUND	;and sound
	STA $1DFC

	DEC $1528,x		;decrement hitpoints
	BPL Alive

LDA #$04	;\
STA $14C8,x	; |
LDA #$2E	; |
STA $1540,x	; | destroy in starburst and puff of smoke
LDA #$08	; |
STA $1DF9	; |
JSL $07FC3B	;/

	LDA #$0B		;\
	STA $1DFB		; |
	LDA #$FF		; | end level
	STA $13C6		; |
	STA $1493		;/
	
Alive
	RTS

NextSprite
	DEY			;next sprite
	BPL CheckLoop
	RTS

Flash
	LDA $15F6,x		    ;load properties
	AND #$F1		    ;erase ccc bits
	PHA			    ;preserve

	LDA $14		    ;load frame counter
	LSR A		    ;/2
	LSR A		    ;/4 (each lsr is a /2)
	AND #$03		    ;this means only values 00-03 get through
	TAY			    ;into Y index

	PLA			    ;restore properties
	ORA PALETTETABLE,y	    ;this will set the ccc portion
	STA $15F6,x		    ;store new properties
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; flame interaction routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


X_OFFSET	    dcb $1A,$14,$EE,$F8			
X_OFFSET2	    dcb $00,$00			


FLAME_INTERACT	    LDA $157C,x             ; \ set direction...
		    ASL A                   ;  |
		    ADC $1602,x             ;  | set frames...
		    TAY                     ; / transfer to...

		    LDA $E4,x               ; \ store x position (Low Byte)
		    CLC                     ;  |
		    ADC X_OFFSET,y          ;  |
		    STA $04                 ; /
		    LDA $14E0,x             ; \ store x position (High Byte)
		    ADC X_OFFSET2,y         ;  |
		    STA $0A                 ; /
		    LDA #$04                ;
		    STA $06                 ;
		    STA $07                 ;

		    LDA $D8,x               ; \ store y position (Low Byte)
		    CLC                     ;  |
		    ADC #$47                ;  |
		    STA $05                 ; /
		    LDA $14D4,x             ; \ store y position (High Byte)
		    ADC #$00                ;  |
		    STA $0B                 ; /

FLAME_CONTACT	    JSL $03B664             ;
		    JSL $03B72B             ; check for contact routine
		    BCC RETURN2             ; return if no contact
		    JSL $00F5B7             ; hurt Mario routine                    
RETURN2		    RTS                     ; return


;A smoke routine
;===============

SUB_SMOKE           LDY #$03                ; \ find a free slot to display effect
FINDFREE            LDA $17C0,y             ;  |
                    BEQ FOUNDONE            ;  |
                    DEY                     ;  |
                    BPL FINDFREE            ;  |
                    RTS                     ; / return if no slots open

FOUNDONE            LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA $D8,x               ; \ smoke y position = generator y position
                    STA $17C4,y             ; /
                    LDA #$18                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $E4,x               ; \ load generator x position and store it for later
                    STA $17C8,y             ; /
                    LDX $15E9
             
                    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; START
PALETTETABLE	    dcb $06,$0A,$04,$08	    ;the ccc portion of yxppccct, THE 't' MUST BE ZERO
;; END

TILEMAP		    dcb $60,$60,$64,$8A,$60,$60,$AC,$AC,$AC,$CE
PROPERTIES          dcb $04,$04,$0D,$09,$04,$04,$0D,$0D,$0D,$07

HORZ_DISP	    dcb $FB,$05,$00,$F2,$FD,$03,$EA,$EA,$EA,$EA
		    dcb $FB,$05,$00,$FA,$FD,$03,$F2,$F2,$F2,$F2
		    dcb $FB,$05,$00,$0E,$03,$FD,$16,$16,$16,$16			
		    dcb $FB,$05,$00,$06,$03,$FD,$0E,$0E,$0E,$0E

VERT_DISP           dcb $0B,$0B,$00,$03,$0F,$0F,$13,$23,$33,$43			
		
FLAME_TILES	    dcb $CC,$CE,$CC,$CE
FLAME_PROPERTIES    dcb $00,$00,$40,$40

CLOUD_PROPERTIES    dcb $00,$40,$C0,$80			


SUB_GFX		    JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
                                            ; $00 = sprite x position relative to screen boarder 
                                            ; $01 = sprite y position relative to screen boarder 
		    LDA $1602,x             ; \ $04 = index to frame start (0 or 1)
		    STA $04                 ; /
                    LDA $157C,x             ; \ $02 = sprite direction
                    STA $02                 ; /
		    PHX                     ; push sprite index

		    PHY                     ; push sprite index
		    LDX #$09                ; loop counter = (number of tiles per frame) - 1

LOOP_START	    LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
		    CLC                     ;  |
		    ADC VERT_DISP,x         ;  |
		    STA $0301,y             ; /

		    STZ $03                 ; restore tilemap direction

		    LDA TILEMAP,x           ; set tilemap
		    CPX #$09                ; \ if it's not a flame tile...
		    BNE STORE_TILEMAP       ; /

		    LDA $14                 ; \ make flame tilemap animated
		    LSR A                   ;  |
		    LSR A                   ;  |
		    PHX                     ;  |
		    AND #$03                ;  |
		    TAX                     ; / transfer to...
 
		    LDA FLAME_PROPERTIES,x  ; \ store properties from flame
		    STA $03                 ; /
		    LDA FLAME_TILES,x       ; \ set flame tiles
		    PLX                     ;  |
STORE_TILEMAP	    STA $0302,y             ; / store tilemap

		    LDA $02                 ; \ if the direction is set...
		    CMP #$01                ; /
		    LDA PROPERTIES,x        ; \ get tile properties
		    EOR $03                 ;  |
		    ORA #$30                 ;  | add in tile priority of level
		    BCS STORE_PROPERTIES    ;  |
		    EOR #$40                ;  | flip tile
STORE_PROPERTIES    STA $0303,y             ; / store tile properties

		    PHX                     ; push x
		    LDA $04                 ; \ if the image is set...
		    BEQ DIRECTION_IS_SET    ; /
		    TXA
		    CLC
		    ADC #$0A
		    TAX

DIRECTION_IS_SET    LDA $02                 ; \ if the direction is set...
		    BNE SET_HORZ_DISP       ; /
		    TXA
		    CLC
		    ADC #$14
		    TAX

SET_HORZ_DISP	    LDA $00                 ; \ tile x position = sprite x location ($00) + tile displacement
		    CLC                     ;  |
		    ADC HORZ_DISP,x         ;  |
		    STA $0300,y             ; /

		    PLX                     ; \ pull, X = current tile of the frame we're drawing
                    INY                     ;  | increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ;  |    ...so increment 4 times
                    DEX                     ;  | go to next tile of frame and loop
                    BPL LOOP_START          ; / 

		    LDA $14                 ; \ make cloud tilemap animated
		    LSR A                   ;  |
		    LSR A                   ;  |
		    LSR A                   ;  |
		    AND #$03                ;  |
		    TAX                     ; /
		    PLY                     ; pull y

		    LDA CLOUD_PROPERTIES,x  ; \ set cloud properties
		    EOR $0313,y             ;  |
		    STA $0313,y             ;  |
		    STA $0327,y             ;  |
		    EOR #$C0                ;  |
		    STA $0317,y             ;  |
		    STA $0323,y             ; /

		    PLX                     ; pull, X = sprite index
		    LDY #$02                ; \ 460 = 2 (all 16x16 tiles)
		    LDA #$09                ;  | A = (number of tiles drawn - 1)
		    JSL $01B7B3             ; / don't draw if offscreen
		    RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

